"""Function to create the savebutton or to clean sur frame_layout (left)"""
from PIL import Image, ImageDraw, ImageFont
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (QPushButton, QFileDialog)
from PyQt5.QtGui import QPainter, QPixmap, QImage
import matplotlib.pyplot as plt
from Plot.style_plot import save_button_style


def ouvrir_image(dirname, numbers_str, x, y, image_raman):
    """
    To open image in the right frame (if image are available)
    """

    dir_raman = rf'{dirname}/{numbers_str}/Spectra/{x}_{y}_fitted.png'
    print(dir_raman)

    try:
        image = Image.open(dir_raman)
        image = image.resize((600, 250))
        draw = ImageDraw.Draw(image)
        font = ImageFont.truetype("arial.ttf", 24)
        draw.text((70, 10), f"Wafer {numbers_str} - {x}/{y}", fill="black",
                  font=font)
        image_data = image.convert("RGB")
        qimage = QImage(image_data.tobytes(), image_data.width,
                        image_data.height, image_data.width * 3,
                        QImage.Format_RGB888)

        img = QPixmap.fromImage(qimage)
        image_raman.setPixmap(img)
    except (FileNotFoundError, IOError):
        print(f"Image {dir_raman} non trouvée ou erreur lors de l'ouverture.")


def create_savebutton(layout, frame_left, frame_right):
    """Create a save button to capture and save specific
    frames as a combined image."""

    def save_image():
        """Capture and save specific frames as a combined image."""
        screen_left = frame_left.grab()
        screen_right = frame_right.grab()

        # Get the file path to save the combined image
        file_name, _ = QFileDialog.getSaveFileName(None,
                                                   "Save Combined Frame Screenshot",
                                                   "",
                                                   "PNG Files (*.png);;All Files (*)")

        if file_name:
            combined_width = screen_left.width() + screen_right.width()
            combined_height = max(screen_left.height(), screen_right.height())
            combined_pixmap = QPixmap(combined_width, combined_height)

            # Fill the combined QPixmap with a white background
            combined_pixmap.fill(Qt.white)

            painter = QPainter(combined_pixmap)
            painter.drawPixmap(0, 0, screen_left)
            painter.drawPixmap(screen_left.width(), 0, screen_right)
            painter.end()

            # Save the combined image
            combined_pixmap.save(file_name, "PNG")

    # Add the save button directly to the layout
    save_button = QPushButton("Screenshot")
    save_button.setStyleSheet(save_button_style())
    save_button.clicked.connect(save_image)
    layout.addWidget(save_button, 8, 0, 1, 7)


def clear_frame(frame_left_layout):
    """Clear all widgets and layouts from the given frame_left_layout
    and reset images and text labels."""
    for i in reversed(range(frame_left_layout.count())):
        item = frame_left_layout.itemAt(i)
        if item.layout():
            clear_layout(item.layout())
            frame_left_layout.removeItem(item)
        elif item.widget():
            widget = item.widget()
            widget.deleteLater()

    # Close any open Matplotlib figures
    for fig in plt.get_fignums():
        plt.close(fig)


def clear_layout(layout):
    """Recursively clear all widgets and sub-layouts inside a given layout."""
    while layout.count():
        item = layout.takeAt(0)
        if item.layout():
            clear_layout(item.layout())
        elif item.widget():
            item.widget().deleteLater()



import os
import json
import sys
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import (
    QWidget, QButtonGroup, QPushButton, QLabel, QGroupBox, QGridLayout, QTableWidget,
    QTableWidgetItem, QVBoxLayout, QHeaderView, QDialog, QFileDialog, QProgressDialog, QHBoxLayout,
    QLineEdit, QRadioButton, QCheckBox, QSizePolicy, QVBoxLayout
)
from PyQt5.QtGui import QFont  # Import QFont to change font size
from PyQt5.QtCore import Qt
from PyQt5.QtCore import pyqtSignal

class SettingsWindow(QDialog):
    data_updated = pyqtSignal()  # Signal emitted when data is updated

    def __init__(self):
        super().__init__()
        self.setWindowTitle("Settings")
        self.resize(1200, 800)

        # Get the user's folder path (C:\Users\XXXXX)
        # This gets the user's home directory
        self.user_folder = os.path.expanduser("~")

        # Define the new folder "SEM" and the file to store settings
        self.new_folder = os.path.join(self.user_folder, "Raman")
        self.data_file = os.path.join(self.new_folder, "settings_data.json")
        
        # Set default settings file path
        self.default_settings_file = r'C:\Users\TM273821\PycharmProjects\Raman_sSOI_V2\RamanSOI\asset\Setting_Si.json'
        
        self.data = []  # Structure to store the table data
        self.input_values = []



        # Set up the UI layout
        self.layout = QHBoxLayout()  # Use QHBoxLayout for side-by-side layout
        self.setLayout(self.layout)

        # Create a container layout for the table and buttons (Vertical layout for table and buttons)
        self.table_layout = QVBoxLayout()

        # Create and add the table to the layout
        self.create_table()

        # Create the input fields section
        self.create_input_fields()

        # Create and add the buttons below the table
        self.create_buttons()

        # Load data from the settings file
        self.load_data()

        # Add the table layout to the main layout
        self.layout.addLayout(self.table_layout)

    def load_json_file(self):
        """Load settings from a JSON file and update settings window."""
        options = QFileDialog.Options()
        file_path, _ = QFileDialog.getOpenFileName(self,
                                                   "Open JSON File",
                                                   "",
                                                   "JSON Files (*.json);;All Files (*)",
                                                   options=options)


        if file_path:
            try :
                with open(file_path, "r") as file:
                    saved_data = json.load(file)
                    print(saved_data)

                    # Réinitialiser la table et les données
                    self.table.setRowCount(0)  # Vider la table
                    self.data = []  # Réinitialiser la structure interne

                    table_data = saved_data.get("table_data", [])
                    for row_data in table_data:
                        self.add_row(
                            row_data.get("Peak", ""), row_data.get("Ylabel", ""),
                            row_data.get("Filename", ""), row_data.get("Min", ""),
                            row_data.get("Max", ""), row_data.get("Threshold", False),
                            row_data.get("Boxplot", False), update_data=False
                        )
                    self.data = table_data

                    # Charger les valeurs des champs d'entrée
                    self.input_values = saved_data.get("input_values")
                    self.wafer_size_input.setText(self.input_values.get("Wafer Size"))
                    self.edge_exclusion_input.setText(
                        self.input_values.get("Edge Exclusion"))
                    self.spectra_xmin_input.setText(
                        self.input_values.get("Spectra xmin"))
                    self.spectra_xmax_input.setText(
                        self.input_values.get("Spectra xmax"))
                    self.spectra_ymin_input.setText(
                        self.input_values.get("Spectra ymin"))
                    self.spectra_ymax_input.setText(
                        self.input_values.get("Spectra ymax"))
                    self.spectra_xlabel_input.setText(self.input_values.get("Xlabel"))
                    self.spectra_ylabel_input.setText(self.input_values.get("Ylabel"))

                    print("Data loaded successfully:", self.data, self.input_values)
            except FileNotFoundError:
                print("No previous data found. Starting fresh.")
                self.data = []
            except Exception as e:
                print(f"Error loading data: {e}")
                self.data = []

    def create_table(self):
        """Create the settings table."""
        self.table = QTableWidget()
        self.table.setColumnCount(7)  # Total 6 columns
        self.table.setHorizontalHeaderLabels(["Peak", "Ylabel", "Filename", "Min", "Max", "Threshold", "Boxplot"])
        self.table.setMinimumHeight(200)
        self.table.setMinimumWidth(800)

        self.table.blockSignals(True)

        # Appliquer un style pour augmenter la taille du texte
        self.table.setStyleSheet("""
               QTableWidget {
                   font-size: 16px;  /* Augmenter la taille du texte */
                   text-align: center;  /* Centrer le texte dans toutes les cellules */
               }
               QTableWidget::item {
                   padding: 5px;  /* Ajouter un peu de marge autour du texte */
                   text-align: center;  /* Centrer le texte dans les cellules */
               }
               QHeaderView::section {
                   font-size: 16px;  /* Taille du texte des en-têtes */
                   background-color: #f2f2f2;  /* Couleur de fond pour les en-têtes */
                   text-align: center;  /* Centrer le texte dans les en-têtes */
               }
           """)

        # Populate the table with existing data
        for row_data in self.data:
            self.add_row(
                row_data.get("Peak", ""), row_data.get("Ylabel", ""),
                row_data.get("Filename", ""), row_data.get("Min", ""),
                row_data.get("Max", ""), row_data.get("Threshold", False),
                row_data.get("Boxplot", False), update_data=False
            )

        self.table.blockSignals(False)
        self.table.itemChanged.connect(self.update_data)
        self.table_layout.addWidget(self.table)


    def remove_selected_row(self):
        """Remove the currently selected row."""
        current_row = self.table.currentRow()
        if current_row != -1:
            self.table.removeRow(current_row)
            del self.data[current_row]


    def create_buttons(self):
        """Create buttons to add and remove rows, and load JSON."""
        add_button = QPushButton("Add Row")
        remove_button = QPushButton("Remove Selected Row")
        load_button = QPushButton("Load JSON")
        save_button = QPushButton(
            "Save JSON File")

        button_style = """
                QPushButton {
                    font-size: 18px;  /* Augmenter la taille de la police */
                    background-color: #f0f0f0;  /* Fond gris clair */
                    color: black;  /* Texte noir */
                    border: 2px solid black;  /* Bordure noire */
                    border-radius: 10px;  /* Coins arrondis */
                    padding: 10px 20px;  /* Espacement interne */
                    min-width: 150px;  /* Largeur minimale pour chaque bouton */
                    transition: all 0.3s ease;  /* Transition douce pour les effets de survol */
                }

                QPushButton:hover {
                    background-color: #e0e0e0;  /* Fond gris légèrement plus foncé au survol */
                    border-color: black;  /* Bordure noire inchangée */
                }

                QPushButton:pressed {
                    background-color: #d0d0d0;  /* Fond gris encore plus foncé lorsqu'on appuie */
                    border-color: black;  /* Bordure noire inchangée */
                }
            """

        # Appliquer le style aux boutons
        add_button.setStyleSheet(button_style)
        remove_button.setStyleSheet(button_style)
        load_button.setStyleSheet(button_style)
        save_button.setStyleSheet(button_style)

        add_button.clicked.connect(self.add_row)
        remove_button.clicked.connect(self.remove_selected_row)
        load_button.clicked.connect(
            self.load_json_file)  # Connect load button to load_json_file
        save_button.clicked.connect(self.save_json_file)

        # Add buttons to the layout under the table
        self.table_layout.addWidget(add_button)
        self.table_layout.addWidget(remove_button)
        self.table_layout.addWidget(
            load_button)  # Add the load button to the layout
        self.table_layout.addWidget(
            save_button)  # Ajouter le bouton "Save" au layout

    def save_json_file(self):
        """Save settings to a JSON file."""
        options = QFileDialog.Options()
        file_path, _ = QFileDialog.getSaveFileName(self,
                                                   "Save JSON File",
                                                   "",
                                                   "JSON Files (*.json);;All Files (*)",
                                                   options=options)
        if file_path:
            # Récupérer les valeurs des champs d'entrée
            input_values = {
                "Wafer Size": self.wafer_size_input.text(),
                "Edge Exclusion": self.edge_exclusion_input.text(),
                "Spectra xmin": self.spectra_xmin_input.text(),
                "Spectra xmax": self.spectra_xmax_input.text(),
                "Spectra ymin": self.spectra_ymin_input.text(),
                "Spectra ymax": self.spectra_ymax_input.text(),
                "Xlabel": self.spectra_xlabel_input.text(),
                "Ylabel": self.spectra_ylabel_input.text()
            }

            # Récupérer les données de la table
            table_data = []
            for row in range(self.table.rowCount()):
                # Récupérer les valeurs des colonnes de texte
                peak = self.table.item(row, 0).text() if self.table.item(row,
                                                                         0) else ""
                ylabel = self.table.item(row, 1).text() if self.table.item(row,
                                                                           1) else ""
                filename = self.table.item(row, 2).text() if self.table.item(
                    row, 2) else ""
                min_value = self.table.item(row, 3).text() if self.table.item(
                    row, 3) else ""
                max_value = self.table.item(row, 4).text() if self.table.item(
                    row, 4) else ""

                # Récupérer les états des cases à cocher
                threshold_checkbox = self.table.cellWidget(row, 5)
                boxplot_checkbox = self.table.cellWidget(row, 6)
                threshold_checked = threshold_checkbox.isChecked() if isinstance(
                    threshold_checkbox, QCheckBox) else False
                boxplot_checked = boxplot_checkbox.isChecked() if isinstance(
                    boxplot_checkbox, QCheckBox) else False

                # Ajouter une ligne complète au tableau de données
                table_data.append({
                    "Peak": peak,
                    "Ylabel": ylabel,
                    "Filename": filename,
                    "Min": min_value,
                    "Max": max_value,
                    "Threshold": threshold_checked,
                    "Boxplot": boxplot_checked
                })

            # Sauvegarder les données dans le fichier JSON
            data_to_save = {
                "table_data": table_data,
                "input_values": input_values
            }

            with open(file_path, 'w') as file:
                json.dump(data_to_save, file, indent=4)
            print("Data saved successfully:", data_to_save)

    def create_input_fields(self):
        """Create input fields for wafer size and spectra limits inside a QGroupBox."""
        # Créer un QGroupBox pour les champs d'entrée
        # Créer une fonction pour ajouter des lignes avec un label et un champ de texte alignés à droite

        self.input_group_box = QGroupBox("Input Fields")
        self.input_layout = QVBoxLayout()

        def create_input_row(label_text, input_widget):
            # Créer un QHBoxLayout pour aligner le label à gauche et le champ à droite
            row_layout = QHBoxLayout()

            # Créer le label
            label = QLabel(label_text)
            label.setStyleSheet("""
                    QLabel {
                        font-size: 18px;
                        color: black;  /* Texte noir */
                        margin-right: 10px;  /* Espacement à droite */
                    }
                """)

            # Créer le champ de texte
            input_widget.setStyleSheet("""
                    QLineEdit {
                        font-size: 18px;  /* Taille de la police plus grande */
                        color: black;  /* Texte noir */
                        padding: 10px;
                        min-width: 200px;  /* Largeur minimum pour les champs de texte */
                    }

                """)

            # Ajouter le label et le champ de texte au QHBoxLayout
            row_layout.addWidget(label)
            row_layout.addWidget(input_widget)

            return row_layout

        # Wafer Size Input
        self.wafer_size_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Wafer Size (cm):", self.wafer_size_input))

        # Edge exclusion Input
        self.edge_exclusion_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Edge Exclusion (cm):", self.edge_exclusion_input))

        # Spectra xmin, xmax, ymin, ymax Inputs
        self.spectra_xmin_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Spectra xmin:", self.spectra_xmin_input))

        self.spectra_xmax_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Spectra xmax:", self.spectra_xmax_input))

        self.spectra_ymin_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Spectra ymin:", self.spectra_ymin_input))

        self.spectra_ymax_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Spectra ymax:", self.spectra_ymax_input))

        self.spectra_xlabel_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Xlabel:", self.spectra_xlabel_input))

        self.spectra_ylabel_input = QLineEdit()
        self.input_layout.addLayout(
            create_input_row("Ylabel:", self.spectra_ylabel_input))

        # Associer le layout au QGroupBox
        self.input_group_box.setLayout(self.input_layout)

        # Ajouter le QGroupBox au layout principal
        self.layout.addWidget(self.input_group_box)

    def add_row(self, peak=None, ylabel="", filename="", min_value="",
                max_value="", is_checked=False, boxplot_checked=False,
                update_data=True):
        """Add a new row to the table with default values."""

        # Normalisation de la valeur 'peak'
        if peak in (None,
                    False):  # Traiter les valeurs None ou False comme des chaînes vides
            peak = ""

        row_position = self.table.rowCount()
        self.table.insertRow(row_position)

        # Créer les éléments pour chaque colonne
        peak_item = QTableWidgetItem(str(peak))
        ylabel_item = QTableWidgetItem(str(ylabel))
        filename_item = QTableWidgetItem(str(filename))
        min_item = QTableWidgetItem(str(min_value))
        max_item = QTableWidgetItem(str(max_value))

        # Ajouter les éléments dans la table
        self.table.setItem(row_position, 0, peak_item)
        self.table.setItem(row_position, 1, ylabel_item)
        self.table.setItem(row_position, 2, filename_item)
        self.table.setItem(row_position, 3, min_item)
        self.table.setItem(row_position, 4, max_item)

        # Ajouter la case à cocher dans la colonne "Threshold"
        checkbox = QCheckBox()
        checkbox.setChecked(bool(is_checked))  # Conversion explicite en booléen
        checkbox.stateChanged.connect(
            lambda state, row=row_position: self.handle_checkbox_state_change(
                state, row))
        self.table.setCellWidget(row_position, 5, checkbox)

        # Ajouter la case à cocher dans la colonne "Boxplot"
        boxplot_checkbox = QCheckBox()
        boxplot_checkbox.setChecked(
            bool(boxplot_checked))  # Conversion explicite en booléen
        boxplot_checkbox.stateChanged.connect(
            lambda state,
                   row=row_position: self.handle_boxplot_checkbox_state_change(
                state, row))
        self.table.setCellWidget(row_position, 6, boxplot_checkbox)

        # Mettre à jour la structure de données interne
        if update_data:
            self.data.append({
                "Peak": peak or "",
                "Ylabel": ylabel or "",
                "Filename": filename or "",
                "Min": min_value or "",
                "Max": max_value or "",
                "Threshold": bool(is_checked),  # S'assurer que c'est un booléen
                "Boxplot": bool(boxplot_checked),
                # S'assurer que c'est un booléen
            })

        self.table.itemChanged.connect(self.update_data)

        print(self.data)

    def handle_boxplot_checkbox_state_change(self, state, current_row):
        """Handle state change for the 'Boxplot' checkbox."""
        is_checked = state == Qt.Checked
        if current_row < len(self.data):
            self.data[current_row]["Boxplot"] = is_checked
            print(f"Boxplot updated for row {current_row}: {is_checked}")

    def handle_checkbox_state_change(self, state, current_row):
        """Ensure only one checkbox is selected at a time."""
        if state == Qt.Checked:
            for row in range(self.table.rowCount()):
                if row != current_row:
                    checkbox = self.table.cellWidget(row, 5)
                    checkbox.blockSignals(True)
                    checkbox.setChecked(False)
                    checkbox.blockSignals(False)

            # Update the data to reflect the single selection
            for i, row_data in enumerate(self.data):
                row_data["Threshold"] = (i == current_row)



    def remove_selected_row(self):
        """Remove the currently selected row from the table."""
        current_row = self.table.currentRow()
        if current_row != -1:
            self.table.removeRow(current_row)
            del self.data[current_row]  # Remove the corresponding data
            print("Row removed:", self.data)  # Debugging

    def update_data(self, item):
        """
        Update the data structure when a table cell value or input field value changes.
        """
        if isinstance(item, QTableWidgetItem):  # Si c'est un élément du tableau
            row = item.row()
            column = item.column()

            if row >= len(self.data):
                return  # Si la ligne n'existe pas dans `self.data`, ne rien faire

            # Mettre à jour la ligne correspondante dans `data`
            if column == 0:
                self.data[row]["Peak"] = item.text()
            elif column == 1:
                self.data[row]["Ylabel"] = item.text()
            elif column == 2:
                self.data[row]["Filename"] = item.text()
            elif column == 3:
                self.data[row]["Min"] = item.text()
            elif column == 4:
                self.data[row]["Max"] = item.text()

            # Afficher pour le débogage
            print(f"Table data updated: {self.data[row]}")

        elif isinstance(item, QLineEdit):  # Si c'est un champ d'entrée
            self.input_values = {
                "Wafer Size": self.wafer_size_input.text(),
                "Edge Exclusion": self.edge_exclusion_input.text(),
                "Spectra xmin": self.spectra_xmin_input.text(),
                "Spectra xmax": self.spectra_xmax_input.text(),
                "Spectra ymin": self.spectra_ymin_input.text(),
                "Spectra ymax": self.spectra_ymax_input.text(),
                "Xlabel": self.spectra_xlabel_input.text(),
                "Ylabel": self.spectra_ylabel_input.text(),
            }

            # Afficher pour le débogage
            print(f"Input fields updated: {self.input_values}")

            # Mettre à jour les colonnes Threshold et Boxplot
        for row in range(self.table.rowCount()):
            if row < len(self.data):
                threshold_checkbox = self.table.cellWidget(row, 5)
                boxplot_checkbox = self.table.cellWidget(row, 6)

                if isinstance(threshold_checkbox, QCheckBox):
                    self.data[row]["Threshold"] = threshold_checkbox.isChecked()
                if isinstance(boxplot_checkbox, QCheckBox):
                    self.data[row]["Boxplot"] = boxplot_checkbox.isChecked()

                # Afficher pour le débogage
                print(f"Checkbox data updated for row {row}: {self.data[row]}")

        # Sauvegarder les données mises à jour
        self.save_data()

        print(self.input_values)

    def closeEvent(self, event):
        """Save data to a file when the dialog is closed."""
        self.save_data()
        # Emit signal to indicate data has been updated
        self.data_updated.emit()
        # Call parent method to handle the close event
        super().closeEvent(event)

    def save_data(self):
        """Save the table data and input field values to a JSON file."""
        os.makedirs(os.path.dirname(self.data_file), exist_ok=True)

        # Récupérer les valeurs des champs d'entrée
        self.input_values = {
            "Wafer Size": self.wafer_size_input.text(),
            "Edge Exclusion": self.edge_exclusion_input.text(),
            "Spectra xmin": self.spectra_xmin_input.text(),
            "Spectra xmax": self.spectra_xmax_input.text(),
            "Spectra ymin": self.spectra_ymin_input.text(),
            "Spectra ymax": self.spectra_ymax_input.text(),
            "Xlabel": self.spectra_xlabel_input.text(),
            "Ylabel": self.spectra_ylabel_input.text()
        }

        # Sauvegarder à la fois les données du tableau et les valeurs des champs d'entrée
        data_to_save = {
            "table_data": self.data,  # Sauvegarder les lignes du tableau
            "input_values": self.input_values
            # Sauvegarder les valeurs des champs d'entrée
        }

        print(data_to_save)
        # Sauvegarder dans le fichier JSON
        try:
            with open(self.data_file, "w") as file:
                json.dump(data_to_save, file, indent=4)
            # print("Data saved successfully.")  # Debugging
        except Exception as e:
            print(f"Error saving data: {e}")


    def load_data(self):
        """Load the table data and input field values from the JSON file."""
        # Try to load from default settings file first
        settings_file = None
        if os.path.exists(self.default_settings_file):
            settings_file = self.default_settings_file
            print(f"Loading default settings from: {settings_file}")
        elif os.path.exists(self.data_file):
            settings_file = self.data_file
            print(f"Loading saved settings from: {settings_file}")
        
        if settings_file:
            try:
                with open(settings_file, "r") as file:
                    saved_data = json.load(file)
                    print(saved_data)
                    table_data = saved_data.get("table_data", [])
                    for row_data in table_data:
                        self.add_row(
                            row_data.get("Peak", ""), row_data.get("Ylabel", ""),
                            row_data.get("Filename", ""), row_data.get("Min", ""),
                            row_data.get("Max", ""), row_data.get("Threshold", False),
                            row_data.get("Boxplot", False), update_data=False
                        )
                    self.data = table_data

                    # Charger les valeurs des champs d'entrée
                    self.input_values = saved_data.get("input_values", {})
                    if self.input_values:
                        self.wafer_size_input.setText(self.input_values.get("Wafer Size", ""))
                        self.edge_exclusion_input.setText(self.input_values.get("Edge Exclusion", ""))
                        self.spectra_xmin_input.setText(self.input_values.get("Spectra xmin", ""))
                        self.spectra_xmax_input.setText(self.input_values.get("Spectra xmax", ""))
                        self.spectra_ymin_input.setText(self.input_values.get("Spectra ymin", ""))
                        self.spectra_ymax_input.setText(self.input_values.get("Spectra ymax", ""))
                        self.spectra_xlabel_input.setText(self.input_values.get("Xlabel", ""))
                        self.spectra_ylabel_input.setText(self.input_values.get("Ylabel", ""))

                print("Data loaded successfully:", self.data, self.input_values)
            except Exception as e:
                print(f"Error loading data: {e}")
                self.data = []
                self.input_values = {}
        else:
            print("No settings file found. Starting fresh.")
            self.data = []
            self.input_values = {}

    def get_table_data(self):
        """Get the current data from the table as a list of dictionaries."""
        if self.input_values is not None:
            print(f"Data : {self.data}")
            return self.data, self.input_values


if __name__ == "__main__":
    app = QApplication(sys.argv)
    settings_window = SettingsWindow()
    settings_window.show()
    sys.exit(app.exec_())




"""
module for the button styles
"""


def checkbox_style_num_slot():
    return """
        QCheckBox {
            font-size: 16px;
            spacing: 0px;
        }
        QCheckBox::indicator {
            width: 20px;
            height: 20px;
        }
        QCheckBox::indicator:checked {
            background-color: #E6E6FA; 
            border: 2px solid black;
        }
        QCheckBox::indicator:unchecked {
            background-color: white;
            border: 2px solid #ccc;
        }
    """

def checkbox_style():
    return """
        QCheckBox {
            font-size: 16px;
            spacing: 0px;
        }
        QCheckBox::indicator {
            width: 20px;
            height: 20px;
        }
        QCheckBox::indicator:checked {
            background-color: #f0ca41; 
            border: 2px solid black;
        }
        QCheckBox::indicator:unchecked {
            background-color: white;
            border: 2px solid #ccc;
        }
    """


def common_radiobutton_style():
    return """
        QRadioButton {
            spacing: 0px;
            font-size: 14px;
        }
        QRadioButton::indicator {
            width: 20px;
            height: 20px;
        }
        QRadioButton::indicator:checked {
            background-color: #f0ca41;
            border: 2px solid black;
        }
        QRadioButton::indicator:unchecked {
            background-color: white;
            border: 2px solid #ccc;
        }
    """


def settings_button_style():
    return """
        QPushButton {
            font-size: 16px;
            background-color: #b3e5fc; 
            border: 2px solid #8c8c8c;
            border-radius: 10px; 
            padding: 10px;
        }
        QPushButton:hover {
            background-color: #64b5f6; 
        }
    """


def open_fitspy_button_style():
    return """
        QPushButton {
            font-size: 16px;
            background-color: #ffcccc;
            border: 2px solid #8c8c8c;
            border-radius: 10px;
            padding: 10px;
        }
        QPushButton:hover {
            background-color: #ff9999;
        }
    """


def run_button_style():
    return """
        QPushButton {
            font-size: 16px;
            background-color: #ffcc80;
            border: 2px solid #8c8c8c;
            border-radius: 10px;
            padding: 5px;
        }
        QPushButton:hover {
            background-color: #ffb74d;
        }
    """


def group_box_style():
    return """
        QGroupBox {
            border: 1px solid black;
            border-radius: 5px;
            margin-top: 10px;
            font-size: 20px;
            font-weight: bold;
        }
        QGroupBox::title {
            font-size: 14px; 
            font-weight: bold;
            subcontrol-origin: margin;
            subcontrol-position: top center;
        }
    """



"""
Function_common Module

This module contains functions and classes which are common to all
characterizations.
"""
import sys
import gc
import os
import shutil
from itertools import cycle, islice
from functools import partial
from concurrent.futures import ProcessPoolExecutor
from matplotlib import rcParams
from PIL import Image
import numpy as np
from Layout.Settings import SettingsWindow
from PyQt5.QtWidgets import QApplication

import pandas as pd
import matplotlib.pyplot as plt

rcParams.update({'figure.autolayout': True})

# Default filename for spectra files
SPECTRA_CSV_FILENAME = "Spectrums.csv"


def process_fitted_file(filepath, input):
    """
    Process a single 'fitted.txt' file and generate a spectrum plot.
    """

    xmin = int(input.get('Spectra xmin', 0)) if input.get(
        'Spectra xmin') else None
    xmax = int(input.get('Spectra xmax', 0)) if input.get(
        'Spectra xmax') else None
    ymin = int(input.get('Spectra ymin', 0)) if input.get(
        'Spectra ymin') else None
    ymax = int(input.get('Spectra ymax', 0)) if input.get(
        'Spectra ymax') else None

    Xlabel = input.get('Xlabel') if input.get(
        'Xlabel') else None

    Ylabel = input.get('Ylabel') if input.get(
        'Ylabel') else None

    data_frame = pd.read_csv(filepath, sep='\t', index_col=0)
    my_colors = list(islice(cycle(['k']), len(data_frame.columns[1:])))
    fig, ax = plt.subplots(figsize=(12, 6))
    ax.tick_params(axis='both', which='major', labelsize=15)

    for i, col in enumerate(data_frame.columns[1:]):
        ax.plot(data_frame.iloc[:, 0], data_frame[col], label=col,
                color=my_colors[i])

    ax.set_xlabel(Xlabel, fontsize=20)
    ax.set_xlim(xmin, xmax)
    ax.set_ylim(ymin, ymax)
    ax.set_ylabel(Ylabel, fontsize=20)

    fitted_name = os.path.splitext(os.path.basename(filepath))[0]
    output_path = os.path.join(os.path.dirname(filepath), f"{fitted_name}.png")
    plt.savefig(output_path, bbox_inches='tight')
    plt.close(fig)
    gc.collect()


def rename_coordinates(x, y):
    """
    Updates the x and y coordinate values based on predefined mappings.

    Parameters:
    x (str): The x-coordinate to rename.
    y (str): The y-coordinate to rename.

    Returns:
    tuple: Updated x and y coordinates.
    """
    original_values = ['8.9', '-8.9', '5.9', '-5.9']
    new_values = ['9.0', '-9.0', '6.0', '-6.0']
    for i, value in enumerate(original_values):
        if x == value:
            x = new_values[i]
        if y == value:
            y = new_values[i]
    return x, y


class Common:
    """
    Common Class
    This class contains all functions.
    """

    def __init__(self, dirname, settings_var, input_var):
        self.dirname = dirname
        self.settings_dataframe = settings_var
        self.input = input_var

    def plot_spectrum(self, wafers):
        """
        Generate spectrum plots from fitted.txt files using multiprocessing.
        """
        # Create the output directory if it doesn't exist
        path = os.path.join(self.dirname, 'Graphe')
        os.makedirs(path, exist_ok=True)

        filepaths = []

        # Convert `wafers` to lowercase for consistent comparisons
        wafers_lower = [str(w).lower() for w in wafers]

        # Walk through the directory tree
        for subdir, _, files in os.walk(self.dirname):
            subdir_name = os.path.basename(subdir).strip().lower()

            # Check if the current directory matches one of the wafers
            if subdir_name in wafers_lower:
                spectra_dir = os.path.join(subdir, "Spectra")

                # Check if the "Spectra" subdirectory exists
                if os.path.exists(spectra_dir) and os.path.isdir(spectra_dir):
                    for file in os.listdir(spectra_dir):
                        # Add only files ending with "fitted.txt"
                        if file.endswith(".txt"):
                            filepaths.append(os.path.join(spectra_dir, file))
                else:
                    print(
                        f"No 'Spectra' directory in {subdir_name}")  #
                    # Debugging statement

        # Print collected file paths for verification
        print(f"Filtered file paths: {filepaths}")

        # Prepare partial function for multiprocessing
        process_partial = partial(process_fitted_file, input=self.input)

        # Use ProcessPoolExecutor for parallel processing
        num_cpu_cores = os.cpu_count()
        max_workers = max(1, num_cpu_cores // 2)

        with ProcessPoolExecutor(max_workers=max_workers) as executor:
            executor.map(process_partial, filepaths)

    def plot_spectrum_sample(self):
        """
        Generate spectrum plots from fitted.txt files using multiprocessing.
        """
        filepaths = []

        # Walk through the directory tree
        for subdir, _, files in os.walk(self.dirname):
            spectra_dir = os.path.join(subdir, "Spectra")

            # Check if the "Spectra" subdirectory exists
            if os.path.exists(spectra_dir) and os.path.isdir(spectra_dir):
                for file in os.listdir(spectra_dir):
                    # Add only files ending with "fitted.txt"
                    if file.endswith(".txt"):
                        filepaths.append(os.path.join(spectra_dir, file))
                else:
                    print(
                        f"No 'Spectra' directory")  #
                    # Debugging statement

        # Print collected file paths for verification
        print(f"Filtered file paths: {filepaths}")

        # Prepare partial function for multiprocessing
        process_partial = partial(process_fitted_file, input=self.input)

        # Use ProcessPoolExecutor for parallel processing
        num_cpu_cores = os.cpu_count()
        max_workers = max(1, num_cpu_cores // 2)

        with ProcessPoolExecutor(max_workers=max_workers) as executor:
            executor.map(process_partial, filepaths)

    def create_image_grid(self, zscale=None):
        """
        Plots an image grid based on selected zscale and material type.
        """
        # Extraire filename from settings
        filenames = [s['Filename'].strip() for s in self.settings_dataframe if
                 s.get('Filename')] + ["Stress"]

        image_options = {
            'Auto': [filename + '.png' for filename in filenames],
            'Identical': [filename + '_ID_scale.png' for filename in filenames],
        }

        image_names = image_options.get(zscale)

        print(image_names)

        # Get sorted subfolders
        def sort_key(subfolder_name):
            parts = subfolder_name.split("\\")
            for part in parts:
                if part.isdigit():
                    return int(part)
            return float('inf')

        subfolders = sorted(
            [f.path for f in os.scandir(self.dirname) if f.is_dir()],
            key=sort_key)

        print(subfolders)

        # Calculate grid dimensions
        num_subfolders = len(subfolders)
        rows = -(-num_subfolders // 5)  # Ceiling division

        # Load images from subfolders
        images_list = []
        for image_name in image_names:
            sub_images = [Image.open(os.path.join(subfolder,
                                                  "Mapping", image_name))
                          for subfolder in subfolders if
                          os.path.exists(os.path.join(subfolder,
                                                      "Mapping", image_name))]
            images_list.append(sub_images)

        # Determine grid dimensions based on image sizes
        image_spacing = 50
        grid_width, grid_height = 0, 0
        for sub_images in images_list:
            for image in sub_images:
                grid_width = max(grid_width, 5 * image.width)
                grid_height = max(grid_height, rows * image.height)

        # Create blank grid images and paste images into the grid
        grid_images = [Image.new('RGB', (grid_width + 250, grid_height + 200),
                                 (255, 255, 255))
                       for _ in image_names]

        for idx, sub_images in enumerate(images_list):
            for i, image in enumerate(sub_images):
                x, y = (i % 5) * (image.width + image_spacing), (i // 5) * (
                        image.height + image_spacing)
                grid_images[idx].paste(image, (x, y))

        # Save merged grid images
        save_path = os.path.join(self.dirname, "Graphe", "Mapping")
        os.makedirs(save_path, exist_ok=True)
        for idx, grid_image in enumerate(grid_images):
            grid_image.save(os.path.join(save_path, f"All_{image_names[idx]}"))

    def reboot(self, subdir_filter, split=False):
        """
        Delete unnecessary files.
        """
        subdir_filter = [str(x).strip().lower() for x in subdir_filter]

        for subdir, _, files in os.walk(self.dirname):
            subdir_name = os.path.basename(subdir).strip().lower()
            print(f"Sub directory : {subdir_name}")
            if subdir_name in subdir_filter:
                for folder in ["Graphe", "Mapping", "Spectra", "raw_data",
                               "Liste_data"]:
                    path = os.path.join(self.dirname, folder)
                    path_2 = os.path.join(subdir, folder)

                    if os.path.exists(path):
                        shutil.rmtree(path)
                        print(f"Deleted: {path}")

                    if os.path.exists(path_2):
                        shutil.rmtree(path_2)
                        print(f"Deleted: {path_2}")

        if split:
            filenames_to_remove = {".csv", ".png", ".txt", ".npy",
                                   "mapping.txt",
                                   "Parameters.txt", "Parameters_stats", ".wdf",
                                   "fitted", "_stats"}
        else:
            filenames_to_remove = {".csv", ".png", ".npy",
                                   "mapping.txt",
                                   "Parameters.txt", "Parameters_stats", ".wdf",
                                   "fitted", "_stats"}

        for subdir, dirs, files in os.walk(self.dirname):
            subdir_name = os.path.basename(subdir).strip().lower()
            if subdir_name in subdir_filter:
                for file in files:
                    filepath = os.path.join(subdir, file)

                    # Skip files ending with "Spectrums.csv"
                    if file.endswith("Spectra.csv") or file.endswith("Spectrums.csv"):
                        print(f"Skipping file: {filepath}")
                        continue

                    # Check for other files to remove
                    if any(file.endswith(extension) or extension in file for
                           extension in filenames_to_remove):
                        try:
                            os.remove(filepath)
                            print(f"Deleted file: {filepath}")
                        except FileNotFoundError:
                            print(
                                f"File not found (already deleted?): "
                                f"{filepath}")
                        except Exception as err:
                            print(
                                f"Error while deleting file {filepath}: {err}")

    def reboot_sample(self):
        """
        Delete unnecessary files.
        """

        filenames_to_remove = {".csv", ".png", ".npy",
                               "mapping.txt",
                               "Parameters.txt", "Parameters_stats", ".wdf",
                               "fitted", "_stats"}

        for subdir, dirs, files in os.walk(self.dirname):
            for file in files:
                filepath = os.path.join(subdir, file)
                if any(file.endswith(extension) or extension in file for
                       extension in filenames_to_remove):
                    try:
                        os.remove(filepath)
                        print(f"Deleted file: {filepath}")
                    except FileNotFoundError:
                        print(
                            f"File not found (already deleted?): "
                            f"{filepath}")
                    except Exception as err:
                        print(
                            f"Error while deleting file {filepath}: {err}")

    def plot_columns(self):
        """
        Creates a plot for each column (ordinate) using the first column as
        the x-axis.

        This function iterates through subdirectories to find
        "data_DP.csv" files,
        extracts the data, and generates scatter plots for each column except
        the first one.

        """

        figure_height = 12
        figure_width = 8
        xtick_rotation = 45

        # Map peaks to Y-axis labels and filenames
        peak_to_label_filename = {
            setting["Peak"]: (setting["Ylabel"], setting["Filename"])
            for setting in self.settings_dataframe
        }

        # Iterate through subdirectories to find "data_DP.csv" files
        for subdir, _, files in os.walk(self.dirname):
            for file in files:
                if file.endswith("data_DP.csv"):
                    graph_path = os.path.join(subdir, "Graphe")
                    os.makedirs(graph_path, exist_ok=True)
                    filepath = os.path.join(subdir, file)
                    data = pd.read_csv(filepath)

                    # Identify the first column as the x-axis
                    x_col = data.columns[0]

                    filtered_data = data.dropna(
                        subset=[x_col] + list(data.columns[1:]))
                    x_values = filtered_data[x_col].astype(str)

                    for col in filtered_data.columns[1:]:
                        if col not in peak_to_label_filename:
                            print(
                                f"Warning: No parameter found for column "
                                f"{col}. Skipping.")
                            continue

                        ylabel, filename = peak_to_label_filename[col]

                        # Create a plot for the column
                        fig, ax = plt.subplots(
                            figsize=(figure_height, figure_width))
                        ax.scatter(x_values, filtered_data[col], marker='s',
                                   color='b', s=50)
                        ax.tick_params(axis='both', which='major', labelsize=15)
                        ax.set_ylabel(ylabel, fontsize=26)
                        plt.xticks(rotation=xtick_rotation)

                        # Save the plot
                        plot_filepath = os.path.join(graph_path,
                                                     f"{filename}.png")
                        plt.savefig(plot_filepath, bbox_inches='tight')
                        plt.close()

    def plot_boxplot(self):
        """
        Create and save a boxplot for each peak based on the data in the CSV
        files.
        The plot is saved in a specific directory and the corresponding data
        is also saved in CSV format.
        """
        # Define paths for saving output files
        path_liste = os.path.join(self.dirname, 'Liste_data')
        if not os.path.exists(path_liste):
            os.makedirs(path_liste)

        figure_height = 12
        figure_width = 8

        path2 = os.path.join(self.dirname, "Graphe", "Boxplot")
        if not os.path.exists(path2):
            os.makedirs(path2)

        taille_df = []
        peaks = [
                    setting['Peak'].strip()
                    for setting in self.settings_dataframe
                    if setting['Ylabel']
                ] + ["Delta", "Stress"]  # Ajout de Delta et Stress

        # Map peaks to their corresponding Ylabels and filenames
        peak_to_label_filename = {
            setting["Peak"]: (setting["Ylabel"], setting["Filename"])
            for setting in self.settings_dataframe
        }

        # Ajout manuel de Delta et Stress
        peak_to_label_filename["Delta"] = ("Delta", "Delta")
        peak_to_label_filename["Stress"] = ("Stress (GPa)", "Stress")

        # Iterate through files and find a valid "data_DP.csv"
        for subdir, _, files in os.walk(self.dirname):
            for file in files:
                filepath = os.path.join(subdir, file)
                if filepath.endswith("data_DP.csv"):
                    data_frame = pd.read_csv(filepath)

                    if data_frame is not None and not data_frame.empty:
                        taille_df = data_frame.shape
                        if len(taille_df) >= 2:
                            # If the dataframe has at least 2 dimensions,
                            # determine column count
                            column_number = taille_df[1] - 2
                            print(column_number)
                        else:
                            print("Error: DataFrame has an invalid format.")
                            return
                    else:
                        print(f"The file {filepath} is empty or invalid.")
                        return

        if not taille_df:
            print("Error: No valid file was found.")
            return

        # Proceed to process peaks
        column_number = taille_df[1] - 2
        print(column_number)

        for peak in peaks:
            ylabel, filename = peak_to_label_filename[peak]
            col_data = {}

            # Iterate through subdirectories to collect data for each peak
            for subdir, _, files in os.walk(self.dirname):
                for file in files:
                    filepath = os.path.join(subdir, file)
                    if filepath.endswith("data_DP.csv"):
                        data_frame = pd.read_csv(filepath)
                        data_frame.columns = data_frame.columns.str.strip()
                        print(data_frame.columns)
                        if not data_frame.empty:
                            # Use subdirectory name as the column name
                            nom_colonne = os.path.basename(subdir)
                            col_data[nom_colonne] = data_frame.loc[:, peak]

            # Create a DataFrame from the collected data
            df_merged = pd.DataFrame(col_data)
            df_merged = df_merged[~np.isnan(df_merged)]  # Remove NaN values

            # Reset index of DataFrame
            df_merged.reset_index(drop=True, inplace=True)

            # Ensure column names are integers for sorting
            df_merged.columns = df_merged.columns.astype(int)

            # Sort columns by index
            df_merged = df_merged.sort_index(axis=1)

            # Replace 0 values with NaN
            df_merged[df_merged == 0] = np.nan

            # Prepare data for the boxplot (removing NaN values)
            data_for_boxplot = [df_merged[col].dropna().values for col in
                                df_merged.columns]
            print(data_for_boxplot)

            # Re-create the DataFrame with cleaned data for the boxplot
            df_merged = pd.DataFrame({
                col: pd.Series(data)
                for col, data in zip(df_merged.columns, data_for_boxplot)
            })

            # Save the cleaned data to a CSV file
            nouveau_fichier = f"Boxplot_{filename}.csv"
            df_merged.to_csv(os.path.join(path_liste, nouveau_fichier))

            # Plot the boxplot
            fig, ax = plt.subplots(figsize=(figure_height, figure_width))
            ax.tick_params(axis='both', which='major', labelsize=15)
            ax.boxplot([df_merged[col].dropna() for col in df_merged.columns],
                       showfliers=False)
            ax.set_xticklabels(df_merged.columns)
            ax.set_xlabel('Wafer', fontsize=26)
            ax.set_ylabel(ylabel, fontsize=26)

            # Save the plot as a PNG image
            plot_filepath = os.path.join(path2, f"{filename}.png")
            plt.savefig(plot_filepath, bbox_inches='tight')
            plt.close()

    def stats(self):
        """
            stats function
            Create Parameters files. Calculate the mean value for each
            .
        """

        path_liste = os.path.join(self.dirname, 'Liste_data')
        if not os.path.exists(path_liste):
            os.makedirs(path_liste)

        filename = "data_DP.csv"
        filename_parameters = 'Parameters.csv'
        for subdir, _, files in os.walk(self.dirname):
            for file in files:
                filepat = subdir + os.sep
                filepath = subdir + os.sep + file
                if filepath.endswith(filename):
                    os.chdir(filepat)
                    data_frame = pd.read_csv(filepath)
                    stat = data_frame.describe()
                    mod_dataframe = stat.drop(
                        ['count', '25%', '50%', '75%'])
                    mod_dataframe.iloc[1, :] = mod_dataframe.iloc[1, :] * 3
                    mod_dataframe = mod_dataframe.rename(
                        index={'std': '3sigma'})
                    mod_dataframe = mod_dataframe.transpose()

                    mod_dataframe.to_csv(filename_parameters)
                    mod_dataframe = mod_dataframe.drop(
                        ['X', 'Y'])

                    slot_number = \
                        os.path.split(os.path.dirname(filepath))[
                            -1]
                    mod_dataframe['Slot'] = slot_number
                    mod_dataframe.to_csv(filename_parameters)

        parameters_dataframe = pd.DataFrame(
            columns=['Unnamed: 0', 'mean', '3sigma', 'min', 'max'])
        for subdir, dirs, files in os.walk(self.dirname):
            for file in files:
                filepat = subdir + os.sep
                filepath = subdir + os.sep + file
                if filepath.endswith(filename_parameters):
                    os.chdir(filepat)
                    data_frame = pd.read_csv(filepath)
                    parameters_dataframe = pd.concat(
                        [parameters_dataframe, data_frame])

        # Reorder columns to have 'Slot' as the first column
        cols = ['Slot'] + [col for col in
                           parameters_dataframe.columns if
                           col != 'Slot']
        parameters_dataframe = parameters_dataframe[cols]

        parameters_dataframe = parameters_dataframe.rename(
            columns={'Unnamed: 0': 'Parameters'})
        parameters_dataframe.to_csv(
            self.dirname + os.sep + "Liste_data" + os.sep + 'Stats.csv',
            index=False)

    def spectra_split(self, subdir_filter):
        """
        Split and process spectra files based on specific criteria.

        Parameters:
        subdir_filter (list): List of subdirectory names to process.
        """
        # Standardize subdirectory filter (lowercase and stripped)
        subdir_filter = [str(x).strip().lower() for x in subdir_filter]

        # Remove files that do not end with the spectra CSV filename
        for file in os.listdir(self.dirname):
            filepath = os.path.join(self.dirname, file)
            if os.path.isfile(filepath) and not \
                    filepath.endswith(SPECTRA_CSV_FILENAME):
                os.remove(filepath)

        all_dirs = []

        # Iterate through subdirectories to move files
        for subdir, _, files in os.walk(self.dirname):
            subdir_name = os.path.basename(subdir).strip().lower()
            print(f"Checking subdir: {subdir_name}")  # Debugging
            if subdir_name not in subdir_filter and subdir != self.dirname:
                print(f"Ignoring subdir: {subdir_name}")
                continue

            for file in files:
                if file.endswith(SPECTRA_CSV_FILENAME):
                    # Extract third part of the filename
                    parts = file.split("_")
                    if len(parts) < 3:
                        print(f"Skipping file {file}: invalid "
                              f"naming convention.")
                        continue
                    third_part = parts[2].lstrip("0")
                    new_subdir = os.path.join(self.dirname, third_part)

                    # Create subdirectory if it does not exist
                    os.makedirs(new_subdir, exist_ok=True)

                    # Move the file to the new subdirectory
                    src_filepath = os.path.join(subdir, file)
                    dest_filepath = os.path.join(new_subdir, file)
                    
                    # Only move if source and destination are different
                    if os.path.abspath(src_filepath) != os.path.abspath(dest_filepath):
                        shutil.move(src_filepath, dest_filepath)
                        print(f"Moved {file} to {new_subdir}.")
                    else:
                        print(f"File {file} already in correct location {new_subdir}.")

                    # Add new subdirectory to the list if it is
                    # not already present
                    if new_subdir not in all_dirs:
                        all_dirs.append(new_subdir)

        # Include subdirectories corresponding to the filter
        for subdir, _, files in os.walk(self.dirname):
            subdir_name = os.path.basename(subdir).strip().lower()
            if subdir_name in subdir_filter and subdir not in all_dirs:
                print(f"Adding subdir: {subdir}")  # Debugging
                all_dirs.append(subdir)

        print("All directories to process:", all_dirs)

        # Process files in the collected directories
        for subdir in all_dirs:
            print(f"Processing subdir: {subdir}")
            for file in os.listdir(subdir):
                filepath = os.path.join(subdir, file)
                if filepath.endswith(SPECTRA_CSV_FILENAME) and "Ref" not in filepath:
                    print(f"Processing file: {filepath}")
                    data_frame = pd.read_csv(filepath, sep=';', skiprows=2,
                                             header=None)
                    for i in range(0, len(data_frame) - 1, 2):
                        df_chunk = data_frame.iloc[i:i + 2]
                        if len(df_chunk) == 2:
                            # Process chunk of data
                            first_col_value = df_chunk.iloc[0, 0] / 10
                            first_col_value = first_col_value.round(1)
                            second_col_value = df_chunk.iloc[0, 1] / 10
                            second_col_value = second_col_value.round(1)

                            spectrum = df_chunk.T.iloc[2:, :]
                            output_filename = f"{first_col_value}_{second_col_value}.txt"
                            output_filepath = os.path.join(subdir,
                                                           output_filename)

                            try:
                                spectrum.to_csv(output_filepath, sep=';',
                                                index=False,
                                                header=False)
                                print(f"Output written to {output_filepath}")
                            except Exception as err:
                                print(
                                    f"Error writing file {output_filepath}: {err}")
                if filepath.endswith("Ref_" + SPECTRA_CSV_FILENAME):
                    print(f"Processing file: {filepath}")
                    data_frame = pd.read_csv(filepath, sep=';', skiprows=2,
                                             header=None)
                    for i in range(0, len(data_frame) - 1, 2):
                        df_chunk = data_frame.iloc[i:i + 2]
                        if len(df_chunk) == 2:

                            spectrum = df_chunk.T.iloc[2:, :]
                            if i==0:
                                output_filename = f"ref_start.txt"
                            if i==2:
                                output_filename = f"ref_end.txt"
                            output_filepath = os.path.join(subdir,
                                                           output_filename)

                            try:
                                spectrum.to_csv(output_filepath, sep=';',
                                                index=False,
                                                header=False)
                                print(f"Output written to {output_filepath}")
                            except Exception as err:
                                print(
                                    f"Error writing file {output_filepath}: {err}")

    def rename(self):
        """
        Renames files based on their x and y coordinates and applies a
        subdirectory filter.
        """
        for root, _, files in os.walk(self.dirname):
            for file in files:
                if file.endswith(".txt"):
                    parts = file.split('_')
                    x = parts[0]
                    y = parts[1].replace(".txt", "")
                    x, y = rename_coordinates(x, y)
                    new_name = f'{x}_{y}.txt'
                    new_path = os.path.join(root, new_name)

                    # Check if the target filename already exists
                    if os.path.exists(new_path):
                        print(
                            f"File already exists: {new_path}, skipping "
                            f"rename.")
                        continue

                    # Rename the file
                    os.rename(os.path.join(root, file), new_path)
                    print(f'{os.path.join(root, file)} -> {new_path}')

    def rename_spectra(self):
        """
        Renames files based on their x and y coordinates and applies a
        subdirectory filter.
        """
        for root, _, files in os.walk(self.dirname):
            if os.path.basename(root) == 'Spectra':
                for file in files:
                    if file.endswith(".png"):
                        parts = file.split('_')
                        x = parts[0]
                        y = parts[1]
                        x, y = rename_coordinates(x, y)
                        new_name = f'{x}_{y}_fitted.png'
                        new_path = os.path.join(root, new_name)

                        # Check if the target filename already exists
                        if os.path.exists(new_path):
                            print(
                                f"File already exists: {new_path}, skipping "
                                f"rename.")
                            continue

                        # Rename the file
                        os.rename(os.path.join(root, file), new_path)
                        print(f'{os.path.join(root, file)} -> {new_path}')


if __name__ == "__main__":
    DIRNAME = r'C:\Users\TM273821\Desktop\Raman\sSOI'
    SUBDIR = [1,3]

    app = QApplication(sys.argv)
    settings_window = SettingsWindow()
    settings_table, input_table = settings_window.get_table_data()
    settings_window.show()
    app.exec_()

    Common = Common(DIRNAME, settings_table, input_table)
    # Common.spectra_split(SUBDIR)
    # Common.rename()
    # Common.plot_columns()
    # Common.stats()
    Common.plot_boxplot()
    # Common.plot_spectrum(SUBDIR)


    # Common.create_image_grid(zscale="Identical")
    # Common.create_image_grid(zscale="Identical")
